function [X, U, F, sys] = int(dt, N, dim, n_int)
% int Creates sdpvars and constraints for chained integrator model
%
% Chained integrators of order N in 2D and 3D.
% For example, N=2 is a double integrator, where
% x = (x,vx,y,vy), u = (ux,uy)
% x_d = vx, vx_d = ux
% y_d = vy, vy_d = uy
%
% Input: 'dt' timestep (sec)
%        'N' length of traj
%        'dim' workspace dimension (2D or 3D)
%        'n_int' order/degree of model
% Output: 'X' sdpvar for state
%         'U' sdpvar for control
%         'F' dynamic constraints
%         'sys' system info
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details
        
if 3 == dim
    nx = 3*n_int;   % num states
    nu = 3;         % num inputs

    Ac = [diag(ones(n_int-1,1),1), zeros(n_int), zeros(n_int);
          zeros(n_int), diag(ones(n_int-1,1),1), zeros(n_int);
          zeros(n_int), zeros(n_int), diag(ones(n_int-1,1),1)];

    Bc = zeros(nx,nu);
    Bc(n_int,1) = 1;
    Bc(2*n_int,2) = 1;
    Bc(3*n_int,3) = 1;

    % Matlab discretization
    tmp = c2d(ss(Ac,Bc,[],[]),dt);
    sys.A = tmp.a;
    sys.B = tmp.b;
    sys.K = dt*zeros(nx,1);

    C = zeros(3,nx);
    C(1,1) = 1;
    C(2,n_int+1) = 1;
    C(3,2*n_int+1) = 1;
    sys.C = C;

    % Control constraints
    Hu = [eye(nu); -eye(nu)];
    Ku = 0.5*[1;1;
              1;1;
              1;1];
    sys.Uset = polytope(Hu,Ku);

    % State constraints
    x = [0, 10];  % m
    y = [0, 10];  % m
    z = [0, 10];  % m
    bnd = 1;    % m/s, m/s^2, m/s^3, etc (general bound)
    Hx = [eye(nx); -eye(nx)];
    bnd = bnd*ones(n_int-1,1);
    Kx = [x(2);   bnd;  y(2);  bnd;  z(2); bnd;
          -x(1);  bnd; -y(1);  bnd; -z(1); bnd];
    sys.Xset = polytope(Hx,Kx);

    % Create system model
    u = sdpvar(repmat(nu,1,N),ones(1,N));
    x = sdpvar(repmat(nx,1,N+1),ones(1,N+1));

    F = [];
    for k = 1:N
        F = [F, x{k+1} == sys.A*x{k} + sys.B*u{k} + sys.K, ...
                 Hx*x{k}<=Kx, Hu*u{k}<= Ku];
    end
    F = [F, Hx*x{k+1}<=Kx ];

    X = reshape([x{:}],(N+1)*nx,1);
    U = reshape([u{:}],N*nu,1);


elseif 2 == dim
    % 2D point-mass integrator
    nx = 2*n_int; % number of states
    nu = 2;         % number of inputs

    Ac = [diag(ones(n_int-1,1),1), zeros(n_int);
          zeros(n_int), diag(ones(n_int-1,1),1)];

    Bc = zeros(nx,nu);
    Bc(n_int,1) = 1;
    Bc(2*n_int,2) = 1;

    % Matlab discretization
    tmp = c2d(ss(Ac,Bc,[],[]),dt);
    sys.A = tmp.a;
    sys.B = tmp.b;
    sys.K = dt*zeros(nx,1);

    C = zeros(2,nx);
    C(1,1) = 1;
    C(2,n_int+1) = 1;
    sys.C = C;

    % Control constraints
    Hu = [eye(nu); -eye(nu)];
    Ku = 0.5*[1;1;
               1;1];
    sys.Uset = polytope(Hu,Ku);

    % State constraints
    x = [0, 10]; % m
    y = [0, 10]; % m
    bnd = 1;    % m/s, m/s^2, m/s^3, etc (general bound)
    Hx = [eye(nx); -eye(nx)];
    Kx = [x(2);   bnd*ones(n_int-1,1);  y(2);  bnd*ones(n_int-1,1);
          -x(1);  bnd*ones(n_int-1,1); -y(1);  bnd*ones(n_int-1,1);];
    sys.Xset = polytope(Hx,Kx);

    % Create system model
    u = sdpvar(repmat(nu,1,N),ones(1,N));
    x = sdpvar(repmat(nx,1,N+1),ones(1,N+1));

    F = [];
    for k = 1:N
        F = [F, x{k+1} == sys.A*x{k} + sys.B*u{k} + sys.K, ...
                 Hx*x{k}<=Kx, Hu*u{k}<= Ku];
    end
    F = [F, Hx*x{k+1}<=Kx ];

    X = reshape([x{:}],(N+1)*nx,1);
    U = reshape([u{:}],N*nu,1);

else
    error('Invalid dimension.');
end

sys.dim = dim;
sys.n_int = n_int;
sys.nx = nx;
sys.nu = nu;

end
